body[data-jp-deck-mode='presenting'] {
  background: var(--jp-layout-color0);
}

body[data-jp-deck-mode='presenting'] #jp-main-content-panel {
  width: 100vw !important;
  height: 100vh !important;
  border: 0 !important;
  padding: 0 !important;
  margin: 0 !important;
  top: 0 !important;
}

body[data-jp-deck-mode='presenting'] #jp-menu-panel,
body[data-jp-deck-mode='presenting'] #jp-top-panel {
  display: none;
}

body[data-jp-deck-mode='presenting'] .jp-SideBar {
  opacity: 0;
  transition: opacity 0.1s;
  z-index: var(--jp-deck-z-lab-chrome);
  background: none;
  border-color: transparent;
  visibility: var(--jp-deck-chrome-visibility);
}

body[data-jp-deck-mode='presenting'] .jp-SideBar.lm-TabBar.jp-mod-left,
body[data-jp-deck-mode='presenting'] .jp-SideBar.lm-TabBar.jp-mod-right {
  border-color: transparent;
}

body[data-jp-deck-mode='presenting'] .jp-SideBar .lm-TabBar-tab:not(.lm-mod-current) {
  background: unset;
  border-color: transparent;
}

body[data-jp-deck-mode='presenting'] #jp-right-stack,
body[data-jp-deck-mode='presenting'] #jp-left-stack {
  opacity: 0.25;
  z-index: var(--jp-deck-z-lab-chrome);
  transition: opacity 0.1s;
  visibility: var(--jp-deck-chrome-visibility);
}

body[data-jp-deck-mode='presenting'] .jp-SideBar:hover,
body[data-jp-deck-mode='presenting'] #jp-right-stack:hover,
body[data-jp-deck-mode='presenting'] #jp-left-stack:hover {
  opacity: 0.9;
  transition: opacity 0.1s;
}

body[data-jp-deck-mode='presenting'] #jp-main-dock-panel {
  padding: 0;
}
