/** heavy ui tweaks **/
.jp-Deck .jp-InputPrompt,
.jp-Deck .jp-OutputPrompt,
.jp-Deck .jp-cell-toolbar {
  display: none;
}

.jp-Deck .jp-NotebookPanel-toolbar {
  position: absolute;
  border: 0;
  opacity: 0;
  bottom: 0;
  left: 0;
  right: 6em;
  height: var(--jp-private-toolbar-height) !important;
  max-height: var(--jp-private-toolbar-height) !important;
  box-shadow: unset;
}

.jp-Deck .jp-NotebookPanel-toolbar:hover {
  opacity: 0.8;
  transition: opacity 0.1s;
}

.jp-Deck.jp-NotebookPanel {
  display: flex;
  flex-direction: column;
  position: fixed;
  left: var(--jp-private-sidebar-tab-width);
  right: var(--jp-private-sidebar-tab-width);
  justify-content: center;
  align-content: center;
  align-items: stretch;
  height: 100vh;
  overflow-y: hidden;
}

.jp-Deck .jp-Notebook {
  flex-shrink: 0;
  gap: 0;
  overflow-y: auto;
  max-height: 100vh;
  margin: 0;
  padding: 0;
  background: transparent;
}

.jp-Deck .jp-Notebook.jp-mod-scrollPastEnd::after {
  display: none;
}

.jp-Deck .jp-Collapser {
  margin-right: 0.25em;
}

/** on-screen real-estate management **/
.jp-Deck .jp-Cell:not(.jp-deck-mod-onscreen) {
  max-height: 0;
  height: 0;
  overflow: hidden;
  pointer-events: none;
}

.jp-Deck .jp-Cell.jp-deck-mod-onscreen {
  flex: 0;
}

.jp-Deck .jp-Cell.jp-deck-mod-onscreen:not(.jp-deck-mod-visible) {
  opacity: 0;
}

.jp-Deck .jp-Cell.jp-deck-mod-onscreen.jp-deck-mod-visible {
  opacity: 1;
  transition: opacity 0.1s;
}

.jp-Deck .jp-deck-mod-layer {
  position: fixed;
}

/** markdown tweaks **/
.jp-Deck .jp-MarkdownOutput {
  border: 0;
}

/** editor **/
.jp-Deck .jp-InputArea-editor,
.jp-Deck .jp-Notebook.jp-mod-editMode .jp-Cell.jp-mod-active .jp-InputArea-editor {
  border: 0;
  box-shadow: unset;
}

/** side-by-side **/
.jp-Deck .jp-mod-sideBySide.jp-Notebook .jp-Notebook-cell {
  margin: 0 !important;
}
