#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['jupyterlab_deck', 'jupyterlab_deck.tests']

package_data = \
{'': ['*'],
 'jupyterlab_deck': ['_d/*',
                     '_d/share/*',
                     '_d/share/jupyter/*',
                     '_d/share/jupyter/labextensions/*',
                     '_d/share/jupyter/labextensions/@deathbeds/*',
                     '_d/share/jupyter/labextensions/@deathbeds/jupyterlab-deck/*',
                     '_d/share/jupyter/labextensions/@deathbeds/jupyterlab-deck/schemas/*',
                     '_d/share/jupyter/labextensions/@deathbeds/jupyterlab-deck/schemas/@deathbeds/*',
                     '_d/share/jupyter/labextensions/@deathbeds/jupyterlab-deck/schemas/@deathbeds/jupyterlab-deck/*',
                     '_d/share/jupyter/labextensions/@deathbeds/jupyterlab-deck/static/*']}

package_dir = \
{'': 'src'}

install_requires = \
['jupyterlab >=3.3,<4', 'jupyterlab-fonts >=2.1.1']

setup(name='jupyterlab-deck',
      version='0.1.3',
      description='A lightweight presentation mode for JupyterLab.',
      author=None,
      author_email='jupyterlab-deck contributors <deathbeds@googlegroups.com>',
      url=None,
      packages=packages,
      package_data=package_data,
      package_dir=package_dir,
      install_requires=install_requires,
      python_requires='>=3.7',
     )
