.jp-Deck-Remote {
  position: absolute;
  right: 0;
  bottom: 0;
  display: flex;
  flex-direction: column;
  z-index: var(--jp-deck-z-remote);
}

.jp-Deck-Remote-directions {
  display: flex;
  flex: 1;
  flex-direction: column;
  align-items: center;
}

.jp-Deck-Remote label {
  color: var(--jp-ui-font-color1);
}

/* stack */
.jp-Deck-Remote .jp-Deck-Remote-WidgetStack {
  display: flex;
  flex-direction: column;
  align-items: end;
  list-style: none;
}

/* buttons */
.jp-Deck-Remote button {
  border: 0;
  background-color: transparent;
  flex: 1;
}

.jp-Deck-Remote .jp-Deck-Remote-WidgetStack li {
  list-style: none;
}

.jp-Deck-Remote .jp-Deck-Remote-WidgetStack button {
  display: flex;
  flex-direction: row;
  align-items: center;
  gap: 0.5em;
}

.jp-Deck-Remote button:not(.jp-mod-disabled) {
  opacity: 0.75;
  cursor: pointer;
}

.jp-Deck-Remote button:not(.jp-mod-disabled) label {
  cursor: pointer;
}

.jp-Deck-Remote button.jp-mod-disabled {
  opacity: 0.1;
}

.jp-Deck-Remote button:hover:not(.jp-mod-disabled) {
  opacity: 1;
  transition: opacity 0.1s;
}

.jp-Deck-Remote-directions > * {
  flex: 1;
}

.jp-Deck-Remote-directions > div {
  display: flex;
  flex-direction: row;
}

.jp-deck-mod-direction-forward {
  transform: rotate(0.25turn);
}

.jp-deck-mod-direction-down {
  transform: rotate(0.5turn);
}

.jp-deck-mod-direction-back {
  transform: rotate(0.75turn);
}
